
%   C
%  / \
% v  v
% S  R
%  \/
%  v
%  W

C = 1; S = 2; R = 3; W = 4; N = 4;
sizes = 2*ones(1,N);
CPDs = cell(1,N);
CPDs{C} = factorMake(C, 2, [0.5 0.5]);
CPDs{R} = factorMake([C R], [2 2], [0.8 0.2 0.2 0.8]);
CPDs{S} = factorMake([C S], [2 2], [0.5 0.9 0.5 0.1]);
CPDs{W} = factorMake([S R W], [2 2 2], [1 0.1 0.1 0.01 0 0.9 0.9 0.99]);
false = 1; true = 2;

%%%%%%%%%%% naive inference

[F, Z] = naiveInfer(CPDs, []);
assert(Z==1);

evW = factorMake(W, sizes(W), [0 1]); % W = state 2 (true)
[F, alpha] = naiveInfer({CPDs{:}, evW}, [S]);
assert(approxeq(F.T(true), 0.4298)) %P(S=2|W=2)
assert(approxeq(alpha, 0.6471)) %P(W=2)

[F, alpha] = naiveInfer({CPDs{:}, evW}, [R]);
assert(approxeq(F.T(true), 0.7079)) % P(R=2|W=2)

evR = factorMake(R, sizes(R), [0 1]); % R = state 2 (true)
[F, alpha] = naiveInfer({CPDs{:}, evW, evR}, [S]);
assert(approxeq(F.T(true), 0.1945)) % P(S=2|W=2,R=2)

% joint queries (see hw1_sprinkler for naive method)

[F, alpha] = naiveInfer({CPDs{:}, evW}, [S R]);
answer1 = [0         0.5702; 
          0.2921    0.1377];
assert(approxeq(F.T, answer1))

evS = factorMake(S, sizes(S), [0 1]); % S = state 2 (true)
[F, alpha] = naiveInfer({CPDs{:}, evS}, [C W]);
answer2 = [0.0683    0.7650
           0.0047    0.1620];
assert(approxeq(F.T, answer2))


%%%%%%%%%%%%%%%%% Variable elimination

[F, Z] = varElim(CPDs, []);
assert(Z==1);

evW = factorMake(W, sizes(W), [0 1]); % W = state 2 (true)
[F, alpha] = varElim({CPDs{:}, evW}, [S]);
assert(approxeq(F.T(true), 0.4298))
assert(approxeq(alpha, 0.6471))

[F, alpha] = varElim({CPDs{:}, evW}, [R]);
assert(approxeq(F.T(true), 0.7079))

evR = factorMake(R, sizes(R), [0 1]); % R = state 2 (true)
[F, alpha] = varElim({CPDs{:}, evW, evR}, [S]);
assert(approxeq(F.T(true), 0.1945))



% joint queries (see hw1_sprinkler for naive method)

[F, alpha] = varElim({CPDs{:}, evW}, [S R]);
answer1 = [0         0.5702; 
          0.2921    0.1377];
assert(approxeq(F.T, answer1))

evS = factorMake(S, sizes(S), [0 1]); % S = state 2 (true)
[F, alpha] = varElim({CPDs{:}, evS}, [C W]);
answer2 = [0.0683    0.7650
           0.0047    0.1620];
assert(approxeq(F.T, answer2))

fprintf('test passed!\n');
